                org scriptarea

REU_FILES       = 128           ;Number of files that can be buffered to REU.
                                ;Affects the memory dedicated to the "directory"
                                ;within the REU. Each file takes 16 bytes in
                                ;the directory, so with the default value it's
                                ;8KB.

NUMWEAPONATTRIB = 21                              ;Number of attributes in the
NUMBULLETATTRIB = 13                              ;definitions (for loading)
NUMITEMATTRIB   = 6
NUMSPECDMGATTRIB = 5

ilp_ea    = $2d
ilp_dep   = $fb
ilp_stack = $100
ilp_basicregs = $30c


;-------------------------------------------------------------------------------
; MW4 New Engine - initialization (discardable)
;-------------------------------------------------------------------------------

entrypoint:     ldx #CONFIG_LENGTH-1
copyconfig:     lda usefastload,x
                sta configcopy,x
                dex
                bpl copyconfig

saveconfig:     lda #<(-CONFIG_LENGTH)
                ldx #>(-CONFIG_LENGTH)
                sta zp_dest_lo
                stx zp_dest_hi
                lda #$00                        ;Filename 0
                jsr makefilename_direct
                lda #<configcopy
                ldx #>configcopy
                jsr savefile
                jsr initreu

initweapons:    ldx #$00
ld_loop:        jsr ld_getbyte
                tay
                bmi ld_loopdone
                jsr dw_common
                jmp ld_loop
ld_loopdone:    inx
                cpx #$04
                bcc ld_loop

initvideo:      lda $d011
                bpl initvideo
                and #$0f
                sta $d011                       ;Blank screen
                lda #$00
                sta $d020
                sta $d01b                       ;Sprites on top of BG
                sta $d017                       ;Sprite Y-expand off
                sta $d415                       ;Filter lowbyte
                sta joystick                    ;Control reset
                sta prevjoy
                sta keypress
                sta keytype
                lda #$08                        ;Set testbit on all channels
                sta $d404
                sta $d404+7
                sta $d404+14
                lda #$ff                        ;Set all sprites multicolor
                sta $d01c
                sta $d001
                sta $d003
                sta $d005
                sta $d007
                sta $d009
                sta $d00b
                sta $d00d
                sta $d00f
                sta $d015                       ;All sprites on and to the bottom
iv_sprwait1:    lda $d011                       ;Wait for one frame
                bmi iv_sprwait1
iv_sprwait2:    lda $d011
                bpl iv_sprwait2
                lda #$00                        ;Sprites off now
                sta $d015
                lda #SPRMC1
                sta $d025                       ;Set sprite multicolor 1
                lda #SPRMC2
                sta $d026                       ;Set sprite multicolor 2
                lda $dd00                       ;Set game videobank
                and #$fc
                sta $dd00
                ldx #$05
ep_stackloop:   pla
                sta $100,x
                dex
                bpl ep_stackloop
                lda #$38
                sta scrollx
                sta scrolly
                lda #<cleartextscreen
                sta krnon_ctsjump+1
                lda #>cleartextscreen
                sta krnon_ctsjump+2

initsprites:    sei
                lda #$34
                sta $01
                lda #$00
                sta rgscr_spronmask+1
                sta frameupdflag
                sta depacksprflag
                sta irqsprstart
                sta irqirqstart
                sta sortsprstart
                sta sortsprend
                sta sortirqstart
                sta sortirqend
                sta numspr
                sta maptemp1
                tax
                lda #LASTSPRITEFRAME-FIRSTSPRITEFRAME   ;Init free sprite count
                sta freesprites
is_copyfontloop:lda menutextchars,x             ;Copy charset to its ingame
                sta textchars,x                 ;place
                lda menutextchars+$100,x
                sta textchars+$100,x
                inx
                bne is_copyfontloop
                ldx #$02
ilp_loop1:      lda ilp_stack,x
                sec
                sbc ilp_basicregs,x
                sta ilp_stack,x
                dex
                bpl ilp_loop1
                ldx #$3f                        ;Make empty spriteframe
is_emptyfrloop: sta $c000+EMPTYFRAME*64,x
N               set 0
                repeat NUMDEPACKSPR             ;Clear depacksprites
                sta $c000+FIRSTDEPACKFRAME*64+64*N,x
N               set N+1
                repend
                dex
                bpl is_emptyfrloop

                ldx #$03
ilp_loop2:      lda ilp_ea,x
                sec
                sbc ilp_eacheck,x
                clc
                adc maptemp1
                sta maptemp1
                dex
                bpl ilp_loop2

                ldx #NUMDEPACKSPR-1
is_dsprloop:    lda #$ff                        ;Reset depacksystem
                sta dsprf,x
                sta dsprprevf,x
                dex
                bpl is_dsprloop
                lda #$36
                sta $01
                cli
                ldx #$00
is_spradrloop:  lda dspr_adrlo,x                 ;Add baseaddress to
                clc                             ;depack-spritepointers
                adc #<depacksprdata
                sta dspr_adrlo,x
                lda dspr_adrhi,x
                adc #>depacksprdata
                sta dspr_adrhi,x
                inx
                cpx #(dspr_adrhi-dspr_adrlo)
                bcc is_spradrloop

                ldx #$0a
ilp_loop3:      dc.b $bd
                dc.b <ilp_dep
                dc.b >ilp_dep
                sec
                sbc ilp_depcheck,x
                clc
                adc maptemp1
                sta maptemp1
                dex
                bpl ilp_loop3

                ldx #NUMSPR
is_loop:        txa                             ;Create initial sprite order
                sta sortorder,x
                lda #$ff
                sta spry,x
                dex
                bpl is_loop
                jsr initscroll

initraster:     sei
                lda #$35
                sta $01
                lda #<raster_idle
                sta $0314
                lda #>raster_idle
                sta $0315
                lda #<raster_panel
                sta $fffe
                lda #>raster_panel
                sta $ffff
                lda #RASTER_PANEL_POS
                sta $d012
                lda #$60
                sta bganimcode
                ldy maptemp1
                bne ilp_fail
                lda #$4c
ilp_fail:       sta es_scriptjump

                lda #$7f                        ;Set timer interrupt off
                sta $dc0d
                lda #$01                        ;Set raster interrupt on
                sta $d01a
                lda $dc0d                       ;Acknowledge timer interrupt
                cli

                jsr cleartextscreen

drawpanel:      lda #UPD_SCORE|UPD_ITEM|UPD_AMMO ;Next time the panel will
                sta panelupdateflag             ;be updated fully, if it's
                lda #$00                        ;the score/textmode
                sta plrprevhp
                sta plrprevbatt
                sta msgtime

                ldx #39
dp_loop1:       lda #$20
                sta textscreen+21*40,x
                sta textscreen+22*40,x
                sta textscreen+23*40,x
                lda #$01
                sta colors+21*40,x
                sta colors+22*40,x
                lda #$09
                sta colors+23*40,x
                dex
                bne dp_loop1

                ldx #29
                stx textscreen+21*40
                inx
                stx textscreen+22*40
                inx
                stx textscreen+23*40
                ldx #62
                stx textscreen+23*40+39
                lda #$09
                sta colors+21*40
                sta colors+22*40
                sta colors+23*40
                sta colors+23*40+39
                ldx #38
dp_loop2:       lda #60
                sta textscreen+23*40,x
                dex
                bne dp_loop2
                ldx #29
                stx textscreen+21*40+8
                stx textscreen+21*40+31
                inx
                stx textscreen+22*40+8
                stx textscreen+22*40+31
                ldx #61
                stx textscreen+23*40+8
                stx textscreen+23*40+31
                lda #$08
                sta colors+22*40+1                ;Weapon indicator colors
                sta colors+22*40+2
                lda #$09
                sta colors+21*40+8
                sta colors+22*40+8
                sta colors+21*40+31
                sta colors+22*40+31
                ldx #33
dp_loop3:       sta colors+21*40,x                ;Healthbar colors
                sta colors+22*40,x
                inx
                cpx #39
                bcc dp_loop3

                lda #"H"-64                       ;Healthbar letters
                sta textscreen+21*40+32
                lda #"A"-64
                sta textscreen+22*40+32
                lda #9
                sta textwindowleft
                lda #31
                sta textwindowright
                jsr cleartextwindow
                jsr updatepanel

                lda usereu
                beq bigfile_skip                ;Skip if no extra-RAM
                ldx #$00                        ;Try to buffer the "big"
                lda #$7f                        ;file
                jsr makefilename
                jsr openfile
                jsr getbyte                     ;Read ID
                bcs bigfile_skip                ;File couldn't be opened
                cmp #"M"                        ;Check for correctness
                bne bigfile_skip2
                jsr getbyte
                cmp #"W"
                bne bigfile_skip2
                lda #<buffertext2
                ldx #>buffertext2
                ldy #MSGTIME_ETERNAL
                jsr printmsgax
                jsr buffertoreu2
                inc bigfilebuffered
bigfile_skip2:  jsr btr_error                    ;"Drain" the big file
bigfile_skip:   jsr resetmessage

                lda #$00                     ;Load first musicfile
                ldx #FILE_MUSIC              ;to see we have the side 2
                jsr makefilename
sidechange_again:
                lda #<musicarea              ;Disk change loop
                ldx #>musicarea
                jsr loadfiledisk
                beq sidechange_ok
                lda #<sidechange_text
                ldx #>sidechange_text
                ldy #MSGTIME_ETERNAL
                jsr printmsgax
                jsr retryprompt_nomessage    ;Wait for fire
                jmp sidechange_again
sidechange_ok:  jsr relocatemusic

                lda bigfilebuffered          ;If already have the big
                bne skipreu                  ;file, don't buffer individual
                                             ;files
                lda usereu                   ;REU/SuperRAM buffering?
                beq skipreu
                lda fastloadstatus
                beq skipmusic
                lda #$01                     ;If fastloader, start title-
                jsr playgametune             ;music now while buffering
                lda #$00                     ;Allow music playback
                sta music+1
skipmusic:      lda #<buffertext
                ldx #>buffertext
                ldy #MSGTIME_ETERNAL
                jsr printmsgax
                lda #$01                     ;Buffer the fastload drivepart
                jsr bufferfile
bufferloop:     lda #FILE_SPRITE                ;Then onwards from sprites
                jsr bufferfile
                bcs skipreu
                inc bufferloop+1
                jmp bufferloop
skipreu:        jsr resetmessage
                jsr menu_inactivatecounter
                lda #SPRF_COMMON                ;Load common sprites
                jsr loadsprites
                ldx #$ff
                txs
                lda #>(mainloop-1)              ;Push mainloop address
                pha
                lda #<(mainloop-1)
                pha
                lda #<SCRIPT_TITLE              ;Init titlescreen & jump to
                ldx #>SCRIPT_TITLE              ;mainloop afterwards (this
                jmp execscript                  ;code is overwritten)

bufferfile:     jsr makefilename_direct
                jsr getcontrols
                lda joystick
                and #JOY_FIRE
                bne bf_abort
                jmp buffertoreu
bf_abort:       pla
                pla
                jmp skipreu

;-------------------------------------------------------------------------------
; INITRASTER
;
; Initiates raster interrupts.
;
; Parameters: -
; Returns: -
; Modifies: A,X
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; INITSPRITES
;
; Inits the sprite-multiplexing system. Must be called before initting raster
; interrupts.
;
; Parameters: -
; Returns: -
; Modifies: A,X
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; INITREU
;
; REU initialization & memory size detection
; If REU isn't detected, tries to detects SuperRAM.
;
; Parameters: -
; Returns: -
; Modifies: A,X,Y
;-------------------------------------------------------------------------------

initreu:        lda usereu
                bne ir_ok
                rts
ir_ok:          ldx #$08
ir_loop:        lda ir_reutbl,x                 ;Load REU registers that
                sta c64base,x                   ;don't need to change
                dex
                bpl ir_loop
                lda reustatus                   ;Read REU status
                lda #$00                        ;Write 0 to REU address 0
                sta reumemtop                   ;Assume there's no REU memory
                jsr ir_testwrite
ir_memtest:     inc reumemtop                   ;REU memory size detection;
                lda reumemtop                   ;Write the bank number to
                jsr ir_testwrite                ;REU memory
                lda #$ff
                sta loadbuffer                  ;Overwrite the byte in C64
                txa                             ;memory
                jsr ir_testread2                ;Read from address 0
                tax
                bne ir_memtestdone              ;If corrupted (nonzero), exit
                jsr ir_testread
                cmp reumemtop                   ;If written value isn't
                bne ir_memtestdone              ;correct, exit
                jmp ir_memtest
ir_memtestdone: lda reumemtop                   ;REU must have at least
                cmp #$02                        ;128KB of working memory
                bcs ir_reuok
                lda #$00
                sta usereu

                bit $d0b2                   ; Check for hardware-regs disabled
                bmi ir_nosram               ; No? then no SCPU
                lda $d27f                   ; Get highest SRAM-Bank+1
                sec                         ; Subtract lowest SRAM-Bank
                sbc $d27d                   ; (both are 0 if no SRAM)
                cmp #$02                    ; at least 128KB free?
                bcc ir_nosram
                ror usereu                  ; indicate SRAM is present
                dc.b $2c
ir_nosram:      lda #$00
                sta reumemtop
ir_reuok:
                rts

ir_testread:    lda reumemtop
ir_testread2:   sta reuaddrhi
                lda #$00
                sta reuaddrlo
                jsr getreubuffer
                lda loadbuffer
                rts

ir_testwrite:   sta screen2
                lda reumemtop
                sta reuaddrhi
                lda #$00
                sta reuaddrlo
                jmp writereubuffer


;-------------------------------------------------------------------------------
; BUFFERTOREU
;
; Buffers a file into REU memory.
;
; Parameters: Filename
; Returns: C=0 successful
;          C=1 error (A holds errorcode, $00 means out of REU memory)
;
; Modifies: A,X,Y
;-------------------------------------------------------------------------------

btr_error:
btr_closefile:  jsr getbyte                     ;Read bytes from the file until
                bcc btr_closefile               ;closed (error code in A)
                rts

buffertoreu:    lda reumemtop                   ;REU detected?
                beq btr_error
                jsr open_skipreu                ;Open without checking REU
                jsr btr_getstartadr
                bcs btr_error
btr_bufferloop: ldx #$02
btr_byteloop:   jsr getbyte
                bcc btr_storeok
                tay                             ;Check for file not found
                bne btr_error
btr_storeok:    sta screen2,x                   ;Store to screen2
                inx
                ldy fileopen                    ;Loop until file closed
                beq btr_end                     ;or buffer full
                txa
                bne btr_byteloop
btr_end:        sty screen2                     ;Make fake "track,sector link"
                dex                             ;because REU buffering mimics
                stx screen2+1                   ;reading disk sectors
                jsr btr_writebuffer
                lda fileopen
                bne btr_bufferloop
btr_lastblock:  lda #$00
                sta screen2+1
                sta screen2+2                   ;Statuscode OK
                jsr btr_writebuffer

                lda reuaddrlo                   ;Update free page pointer
                sta reufreepage
                lda reuaddrhi
                sta reufreepage+1

                lda reufiles                    ;Get address for dir.entry
                ldy #$04
                ldx #tempadrlo
                jsr mulu
                lda tempadrhi
                sta reuaddrlo
                clc
                adc #>map
                sta tempadrhi
                ldy #$00
                lda filename                    ;Store filename
                sta (tempadrlo),y
                iny
                lda filename+1
                sta (tempadrlo),y
                iny
btr_filestartlo:lda #$00                        ;Store file startaddress
                sta (tempadrlo),y
                iny
btr_filestarthi:lda #$00
                sta (tempadrlo),y
                ldy #$00
                tya
                sta tempadrlo
                sta reuaddrhi
btr_copydirpage:lda (tempadrlo),y               ;Copy directory page to
                sta screen2,y                   ;screen2 for writing
                iny
                bne btr_copydirpage
                jsr writereubuffer

                inc reufiles                    ;Increase number of files
                clc                             ;Return with no error
                rts

btr_writebuffer:lda reuaddrhi
                cmp reumemtop
                bcs btr_wberror
                jmp writereubuffer
btr_wberror:    pla
                pla
                jmp btr_error

btr_getstartadr:lda reufreepage                 ;Get starting address in REU
                sta reuaddrlo                   ;memory for this file
                sta btr_filestartlo+1
                lda reufreepage+1
                sta reuaddrhi
                sta btr_filestarthi+1
                cmp reumemtop                   ;Out of REU memory?
                rts

;-------------------------------------------------------------------------------
; BUFFERTOREU2
;
; Buffers the big datafile into REU memory.
;
; Parameters: -
; Returns:
;
; Modifies: A,X,Y
;-------------------------------------------------------------------------------

btr2_end:       rts
buffertoreu2:   jsr getbyte                     ;Get filename
                bcs btr2_end
                sta filename
                jsr getbyte
                sta filename+1

                jsr getbyte                     ;Get file byte count
                sta alo
                jsr getbyte
                sta ahi

                jsr btr_getstartadr
                bcs btr2_end                    ;Out of memory
btr2_load:      ldx #$02
btr2_loadloop:
                jsr getbyte                     ;Bail out on error/EOF
                bcs btr2_end
btr2_storeok:   sta screen2,x
                inx
                lda alo
                bne btr2_nohigh
                dec ahi
btr2_nohigh:    dec alo                         ;Check for bytecount zero
                lda alo
                ora ahi
                beq btr2_loadend
                cpx #$00
                bne btr2_loadloop
btr2_loadend:   sta screen2
                dex
                stx screen2+1
                jsr btr_writebuffer             ;Write the buffer
                lda screen2
                bne btr2_load                   ;Last buffer?
                jsr btr_lastblock               ;Write endbuffer and
                jmp buffertoreu2                ;move on





;-------------------------------------------------------------------------------
; WRITEREUBUFFER
;
; Writes 256 bytes from screen2 to the REU/SuperRAM
;
; Parameters: reuaddrlo,reuaddrhi:REU address bits 8-23
; Returns:
;          reuaddrlo,reuaddrhi incremented for next page
; Modifies: A,X,Y
;-------------------------------------------------------------------------------


writereubuffer: ldx #$00
                ldy reuaddrlo
                lda reuaddrhi             ;Get REU-pointers
                inc reuaddrlo             ;Increment them for the next
                bne grskip                ;block
                inc reuaddrhi
grskip:         bit usereu                ;REU or SRAM?
                bmi sramsave
                sty reubase+1             ;Set REU-registers
                sta reubase+2
                lda #>screen2
                sta c64base+1
                jsr waitraster
                lda #%10110000            ;Write
                sta command
                lda #>loadbuffer          ;Restore loadbuffer destination
                sta c64base+1             ;for REU reads
                rts

sramsave:       clc
                adc $d27d                   ; add first available SRAM-bank
                sty srammod1+2
                sta srammod1+3
sramloop:       lda screen2,x
srammod1:       dc.b $9f,$00,$00,$02         ; sta $020000,x
                inx
                bne sramloop
                rts



dw_common:      lda dwlotbl,x
                sta alo
                lda dwhitbl,x
                sta ahi
                lda dwnumtbl,x
                sta temp1
                lda dwaddtbl,x
                sta temp2
dw_loop:        jsr ld_getbyte
                sta (alo),y
                dec temp1
                beq dw_ready
                lda alo
                clc
                adc temp2
                sta alo
                bcc dw_loop
                inc ahi
                bne dw_loop
dw_ready:       rts

ld_getbyte:     lda wpndefs
                inc ld_getbyte+1
                bne ld_gbok
                inc ld_getbyte+2
ld_gbok:        rts

dwlotbl:        dc.b <wpn_noisetbl, <blt_xmodtbllo, <itemaddtbl, <specdmgcondition
dwhitbl:        dc.b >wpn_noisetbl, >blt_xmodtbllo, >itemaddtbl, >specdmgcondition
dwnumtbl:       dc.b NUMWEAPONATTRIB, NUMBULLETATTRIB, NUMITEMATTRIB, NUMSPECDMGATTRIB
dwaddtbl:       dc.b NUMWEAPONS, NUMBULLETS, NUMITEMS, NUMSPECDMG

;-------------------------------------------------------------------------------
; REU variables needed only when storing data to the REU.
;-------------------------------------------------------------------------------

ir_reutbl:      dc.b <loadbuffer, >loadbuffer ;C64 base
                dc.b 0,0,0                    ;REU base
                dc.w 256                      ;Transfer length
                dc.b 0                        ;IRQ mask
                dc.b 0                        ;Control register

reufreepage:    dc.w REU_FILES/64
reumemtop:      dc.b $00

bigfilebuffered:dc.b 0
configcopy:     ds.b CONFIG_LENGTH,0

buffertext:     dc.b "BUFFERING TO EXTRA RAM "
                dc.b "- PRESS FIRE TO STOP",0

buffertext2:    dc.b "BUFFERING ALL DATA "
                dc.b "- PLEASE WAIT",0

sidechange_text:
                dc.b "PLEASE INSERT DISK SIDE 2 AND PRESS FIRE",0

ilp_eacheck:    dc.b $a4,$0f,$a4,$0f

ilp_depcheck:   dc.b $01,$00,$01,$80,$c7,$00,$00,$00,$01,$00,$11

initend:
                if initend > musicarea
                err
                endif
